@@ -1,7 +1,6 @@
 package libpod
 
 import (
-	"bytes"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -18,7 +17,6 @@ import (
 const (
 	idRegistryName    = "id-registry"
 	nameRegistryName  = "name-registry"
-	nsRegistryName    = "ns-registry"
 	ctrName           = "ctr"
 	allCtrsName       = "all-ctrs"
 	podName           = "pod"
@@ -40,7 +38,6 @@ const (
 	netNSName          = "netns"
 	containersName     = "containers"
 	podIDName          = "pod-id"
-	namespaceName      = "namespace"
 	networksName       = "networks"
 
 	staticDirName   = "static-dir"
@@ -55,7 +52,6 @@ const (
 var (
 	idRegistryBkt      = []byte(idRegistryName)
 	nameRegistryBkt    = []byte(nameRegistryName)
-	nsRegistryBkt      = []byte(nsRegistryName)
 	ctrBkt             = []byte(ctrName)
 	allCtrsBkt         = []byte(allCtrsName)
 	podBkt             = []byte(podName)
@@ -78,7 +74,6 @@ var (
 	netNSKey      = []byte(netNSName)
 	containersBkt = []byte(containersName)
 	podIDKey      = []byte(podIDName)
-	namespaceKey  = []byte(namespaceName)
 
 	staticDirKey   = []byte(staticDirName)
 	tmpDirKey      = []byte(tmpDirName)
@@ -298,14 +293,6 @@ func getNamesBucket(tx *bolt.Tx) (*bolt.Bucket, error) {
 	return bkt, nil
 }
 
-func getNSBucket(tx *bolt.Tx) (*bolt.Bucket, error) {
-	bkt := tx.Bucket(nsRegistryBkt)
-	if bkt == nil {
-		return nil, fmt.Errorf("namespace registry bucket not found in DB: %w", define.ErrDBBadConfig)
-	}
-	return bkt, nil
-}
-
 func getCtrBucket(tx *bolt.Tx) (*bolt.Bucket, error) {
 	bkt := tx.Bucket(ctrBkt)
 	if bkt == nil {
@@ -400,13 +387,6 @@ func (s *BoltState) getContainerConfigFromDB(id []byte, config *ContainerConfig,
 		return fmt.Errorf("container %s not found in DB: %w", string(id), define.ErrNoSuchCtr)
 	}
 
-	if s.namespaceBytes != nil {
-		ctrNamespaceBytes := ctrBkt.Get(namespaceKey)
-		if !bytes.Equal(s.namespaceBytes, ctrNamespaceBytes) {
-			return fmt.Errorf("cannot retrieve container %s as it is part of namespace %q and we are in namespace %q: %w", string(id), string(ctrNamespaceBytes), s.namespace, define.ErrNSMismatch)
-		}
-	}
-
 	configBytes := ctrBkt.Get(configKey)
 	if configBytes == nil {
 		return fmt.Errorf("container %s missing config key in DB: %w", string(id), define.ErrInternal)
@@ -520,13 +500,6 @@ func (s *BoltState) getPodFromDB(id []byte, pod *Pod, podBkt *bolt.Bucket) error
 		return fmt.Errorf("pod with ID %s not found: %w", string(id), define.ErrNoSuchPod)
 	}
 
-	if s.namespaceBytes != nil {
-		podNamespaceBytes := podDB.Get(namespaceKey)
-		if !bytes.Equal(s.namespaceBytes, podNamespaceBytes) {
-			return fmt.Errorf("cannot retrieve pod %s as it is part of namespace %q and we are in namespace %q: %w", string(id), string(podNamespaceBytes), s.namespace, define.ErrNSMismatch)
-		}
-	}
-
 	podConfigBytes := podDB.Get(configKey)
 	if podConfigBytes == nil {
 		return fmt.Errorf("pod %s is missing configuration key in DB: %w", string(id), define.ErrInternal)
@@ -605,11 +578,6 @@ func (s *BoltState) getVolumeFromDB(name []byte, volume *Volume, volBkt *bolt.Bu
 // Add a container to the DB
 // If pod is not nil, the container is added to the pod as well
 func (s *BoltState) addContainer(ctr *Container, pod *Pod) error {
-	if s.namespace != "" && s.namespace != ctr.config.Namespace {
-		return fmt.Errorf("cannot add container %s as it is in namespace %q and we are in namespace %q: %w",
-			ctr.ID(), s.namespace, ctr.config.Namespace, define.ErrNSMismatch)
-	}
-
 	// Set the original networks to nil. We can save some space by not storing it in the config
 	// since we store it in a different mutable bucket anyway.
 	configNetworks := ctr.config.Networks
@@ -629,11 +597,6 @@ func (s *BoltState) addContainer(ctr *Container, pod *Pod) error {
 	ctrID := []byte(ctr.ID())
 	ctrName := []byte(ctr.Name())
 
-	var ctrNamespace []byte
-	if ctr.config.Namespace != "" {
-		ctrNamespace = []byte(ctr.config.Namespace)
-	}
-
 	// make sure to marshal the network options before we get the db lock
 	networks := make(map[string][]byte, len(configNetworks))
 	for net, opts := range configNetworks {
@@ -670,11 +633,6 @@ func (s *BoltState) addContainer(ctr *Container, pod *Pod) error {
 			return err
 		}
 
-		nsBucket, err := getNSBucket(tx)
-		if err != nil {
-			return err
-		}
-
 		ctrBucket, err := getCtrBucket(tx)
 		if err != nil {
 			return err
@@ -710,12 +668,6 @@ func (s *BoltState) addContainer(ctr *Container, pod *Pod) error {
 			if podCtrs == nil {
 				return fmt.Errorf("pod %s does not have a containers bucket: %w", pod.ID(), define.ErrInternal)
 			}
-
-			podNS := podDB.Get(namespaceKey)
-			if !bytes.Equal(podNS, ctrNamespace) {
-				return fmt.Errorf("container %s is in namespace %s and pod %s is in namespace %s: %w",
-					ctr.ID(), ctr.config.Namespace, pod.ID(), pod.config.Namespace, define.ErrNSMismatch)
-			}
 		}
 
 		// Check if we already have a container with the given ID and name
@@ -744,11 +696,6 @@ func (s *BoltState) addContainer(ctr *Container, pod *Pod) error {
 		if err := namesBucket.Put(ctrName, ctrID); err != nil {
 			return fmt.Errorf("adding container %s name (%s) to DB: %w", ctr.ID(), ctr.Name(), err)
 		}
-		if ctrNamespace != nil {
-			if err := nsBucket.Put(ctrID, ctrNamespace); err != nil {
-				return fmt.Errorf("adding container %s namespace (%q) to DB: %w", ctr.ID(), ctr.Namespace(), err)
-			}
-		}
 		if err := allCtrsBucket.Put(ctrID, ctrName); err != nil {
 			return fmt.Errorf("adding container %s to all containers bucket in DB: %w", ctr.ID(), err)
 		}
@@ -764,11 +711,6 @@ func (s *BoltState) addContainer(ctr *Container, pod *Pod) error {
 		if err := newCtrBkt.Put(stateKey, stateJSON); err != nil {
 			return fmt.Errorf("adding container %s state to DB: %w", ctr.ID(), err)
 		}
-		if ctrNamespace != nil {
-			if err := newCtrBkt.Put(namespaceKey, ctrNamespace); err != nil {
-				return fmt.Errorf("adding container %s namespace to DB: %w", ctr.ID(), err)
-			}
-		}
 		if pod != nil {
 			if err := newCtrBkt.Put(podIDKey, []byte(pod.ID())); err != nil {
 				return fmt.Errorf("adding container %s pod to DB: %w", ctr.ID(), err)
@@ -814,11 +756,6 @@ func (s *BoltState) addContainer(ctr *Container, pod *Pod) error {
 				return fmt.Errorf("container %s depends on container %s which is in a pod - containers not in pods cannot depend on containers in pods: %w", ctr.ID(), dependsCtr, define.ErrInvalidArg)
 			}
 
-			depNamespace := depCtrBkt.Get(namespaceKey)
-			if !bytes.Equal(ctrNamespace, depNamespace) {
-				return fmt.Errorf("container %s in namespace %q depends on container %s in namespace %q - namespaces must match: %w", ctr.ID(), ctr.config.Namespace, dependsCtr, string(depNamespace), define.ErrNSMismatch)
-			}
-
 			depCtrDependsBkt := depCtrBkt.Bucket(dependenciesBkt)
 			if depCtrDependsBkt == nil {
 				return fmt.Errorf("container %s does not have a dependencies bucket: %w", dependsCtr, define.ErrInternal)
@@ -880,11 +817,6 @@ func (s *BoltState) removeContainer(ctr *Container, pod *Pod, tx *bolt.Tx) error
 		return err
 	}
 
-	nsBucket, err := getNSBucket(tx)
-	if err != nil {
-		return err
-	}
-
 	allCtrsBucket, err := getAllCtrsBucket(tx)
 	if err != nil {
 		return err
@@ -919,17 +851,6 @@ func (s *BoltState) removeContainer(ctr *Container, pod *Pod, tx *bolt.Tx) error
 		return fmt.Errorf("no container with ID %s found in DB: %w", ctr.ID(), define.ErrNoSuchCtr)
 	}
 
-	// Compare namespace
-	// We can't remove containers not in our namespace
-	if s.namespace != "" {
-		if s.namespace != ctr.config.Namespace {
-			return fmt.Errorf("container %s is in namespace %q, does not match our namespace %q: %w", ctr.ID(), ctr.config.Namespace, s.namespace, define.ErrNSMismatch)
-		}
-		if pod != nil && s.namespace != pod.config.Namespace {
-			return fmt.Errorf("pod %s is in namespace %q, does not match out namespace %q: %w", pod.ID(), pod.config.Namespace, s.namespace, define.ErrNSMismatch)
-		}
-	}
-
 	if podDB != nil && pod != nil {
 		// Check if the container is in the pod, remove it if it is
 		podCtrs := podDB.Bucket(containersBkt)
@@ -993,9 +914,6 @@ func (s *BoltState) removeContainer(ctr *Container, pod *Pod, tx *bolt.Tx) error
 	if err := namesBucket.Delete(ctrName); err != nil {
 		return fmt.Errorf("deleting container %s name in DB: %w", ctr.ID(), err)
 	}
-	if err := nsBucket.Delete(ctrID); err != nil {
-		return fmt.Errorf("deleting container %s namespace in DB: %w", ctr.ID(), err)
-	}
 	if err := allCtrsBucket.Delete(ctrID); err != nil {
 		return fmt.Errorf("deleting container %s from all containers bucket in DB: %w", ctr.ID(), err)
 	}
@@ -1055,14 +973,11 @@ func (s *BoltState) removeContainer(ctr *Container, pod *Pod, tx *bolt.Tx) error
 
 // lookupContainerID retrieves a container ID from the state by full or unique
 // partial ID or name.
-// NOTE: the retrieved container ID namespace may not match the state namespace.
-func (s *BoltState) lookupContainerID(idOrName string, ctrBucket, namesBucket, nsBucket *bolt.Bucket) ([]byte, error) {
+func (s *BoltState) lookupContainerID(idOrName string, ctrBucket, namesBucket *bolt.Bucket) ([]byte, error) {
 	// First, check if the ID given was the actual container ID
 	ctrExists := ctrBucket.Bucket([]byte(idOrName))
 	if ctrExists != nil {
 		// A full container ID was given.
-		// It might not be in our namespace, but this will be handled
-		// the callers.
 		return []byte(idOrName), nil
 	}
 
@@ -1092,14 +1007,6 @@ func (s *BoltState) lookupContainerID(idOrName string, ctrBucket, namesBucket, n
 	// Search for partial ID matches.
 	exists := false
 	err := ctrBucket.ForEach(func(checkID, checkName []byte) error {
-		// If the container isn't in our namespace, we
-		// can't match it
-		if s.namespaceBytes != nil {
-			ns := nsBucket.Get(checkID)
-			if !bytes.Equal(ns, s.namespaceBytes) {
-				return nil
-			}
-		}
 		if strings.HasPrefix(string(checkID), idOrName) {
 			if exists {
 				return fmt.Errorf("more than one result for container ID %s: %w", idOrName, define.ErrCtrExists)
