@@ -414,7 +414,6 @@ func (r *Runtime) setupContainer(ctx context.Context, ctr *Container) (_ *Contai
 
 		// NewFromSpec() is deprecated according to its comment
 		// however the recommended replace just causes a nil map panic
-		//nolint:staticcheck
 		g := generate.NewFromSpec(ctr.config.Spec)
 		g.RemoveMount("/dev/shm")
 		ctr.config.ShmDir = ""
@@ -477,7 +476,7 @@ func (r *Runtime) setupContainer(ctx context.Context, ctr *Container) (_ *Contai
 			vol.Name = stringid.GenerateRandomID()
 			isAnonymous = true
 		} else {
-			// Check if it exists already
+			// Check if it already exists
 			dbVol, err := r.state.Volume(vol.Name)
 			if err == nil {
 				ctrNamedVolumes = append(ctrNamedVolumes, dbVol)
@@ -496,14 +495,17 @@ func (r *Runtime) setupContainer(ctx context.Context, ctr *Container) (_ *Contai
 		logrus.Debugf("Creating new volume %s for container", vol.Name)
 
 		// The volume does not exist, so we need to create it.
-		volOptions := []VolumeCreateOption{WithVolumeName(vol.Name)}
+		volOptions := []VolumeCreateOption{
+			WithVolumeName(vol.Name),
+			WithVolumeMountLabel(ctr.MountLabel()),
+		}
 		if isAnonymous {
 			volOptions = append(volOptions, withSetAnon())
 		}
 
 		needsChown := true
 
-		// If volume-opts are set parse and add driver opts.
+		// If volume-opts are set, parse and add driver opts.
 		if len(vol.Options) > 0 {
 			isDriverOpts := false
 			driverOpts := make(map[string]string)
@@ -772,16 +774,6 @@ func (r *Runtime) removeContainer(ctx context.Context, c *Container, force, remo
 		cleanupErr = fmt.Errorf("cleaning up container %s: %w", c.ID(), err)
 	}
 
-	// Set ContainerStateRemoving
-	c.state.State = define.ContainerStateRemoving
-
-	if err := c.save(); err != nil {
-		if cleanupErr != nil {
-			logrus.Errorf(err.Error())
-		}
-		return fmt.Errorf("unable to set container %s removing state in database: %w", c.ID(), err)
-	}
-
 	// Remove all active exec sessions
 	// removing the exec sessions might temporarily unlock the container's lock.  Using it
 	// after setting the state to ContainerStateRemoving will prevent that the container is
@@ -794,6 +786,20 @@ func (r *Runtime) removeContainer(ctx context.Context, c *Container, force, remo
 		}
 	}
 
+	// Set ContainerStateRemoving as an intermediate state (we may get
+	// killed at any time) and save the container.
+	c.state.State = define.ContainerStateRemoving
+
+	if err := c.save(); err != nil {
+		if !errors.Is(err, define.ErrCtrRemoved) {
+			if cleanupErr == nil {
+				cleanupErr = err
+			} else {
+				logrus.Errorf("Saving container: %v", err)
+			}
+		}
+	}
+
 	// Stop the container's storage
 	if err := c.teardownStorage(); err != nil {
 		if cleanupErr == nil {
@@ -805,7 +811,7 @@ func (r *Runtime) removeContainer(ctx context.Context, c *Container, force, remo
 
 	// Remove the container's CID file on container removal.
 	if cidFile, ok := c.config.Spec.Annotations[define.InspectAnnotationCIDFile]; ok {
-		if err := os.Remove(cidFile); err != nil {
+		if err := os.Remove(cidFile); err != nil && !errors.Is(err, os.ErrNotExist) {
 			if cleanupErr == nil {
 				cleanupErr = err
 			} else {
