@@ -12,7 +12,6 @@ import (
 	"errors"
 	"fmt"
 	"io/fs"
-	"io/ioutil"
 	"net"
 	"net/http"
 	"net/url"
@@ -30,19 +29,24 @@ import (
 	"github.com/containers/podman/v4/pkg/rootless"
 	"github.com/containers/podman/v4/utils"
 	"github.com/containers/storage/pkg/homedir"
+	"github.com/containers/storage/pkg/ioutils"
 	"github.com/digitalocean/go-qemu/qmp"
 	"github.com/docker/go-units"
 	"github.com/sirupsen/logrus"
 )
 
 var (
-	qemuProvider = &Provider{}
 	// vmtype refers to qemu (vs libvirt, krun, etc).
-	vmtype = "qemu"
+	// Could this be moved into  Provider
+	vmtype = machine.QemuVirt
 )
 
-func GetQemuProvider() machine.Provider {
-	return qemuProvider
+func GetVirtualizationProvider() machine.VirtProvider {
+	return &Virtualization{
+		artifact:    machine.Qemu,
+		compression: machine.Xz,
+		format:      machine.Qcow,
+	}
 }
 
 const (
@@ -65,7 +69,7 @@ const (
 
 // NewMachine initializes an instance of a virtual machine based on the qemu
 // virtualization.
-func (p *Provider) NewMachine(opts machine.InitOptions) (machine.VM, error) {
+func (p *Virtualization) NewMachine(opts machine.InitOptions) (machine.VM, error) {
 	vmConfigDir, err := machine.GetConfDir(vmtype)
 	if err != nil {
 		return nil, err
@@ -219,7 +223,7 @@ func migrateVM(configPath string, config []byte, vm *MachineVM) error {
 	}
 	// Write the config file
 	if err := vm.writeConfig(); err != nil {
-		// If the config file fails to be written, put the origina
+		// If the config file fails to be written, put the original
 		// config file back before erroring
 		if renameError := os.Rename(configPath+".orig", configPath); renameError != nil {
 			logrus.Warn(renameError)
@@ -232,7 +236,7 @@ func migrateVM(configPath string, config []byte, vm *MachineVM) error {
 
 // LoadVMByName reads a json file that describes a known qemu vm
 // and returns a vm instance
-func (p *Provider) LoadVMByName(name string) (machine.VM, error) {
+func (p *Virtualization) LoadVMByName(name string) (machine.VM, error) {
 	vm := &MachineVM{Name: name}
 	vm.HostUser = machine.HostUser{UID: -1} // posix reserves -1, so use it to signify undefined
 	if err := vm.update(); err != nil {
@@ -253,10 +257,12 @@ func (v *MachineVM) Init(opts machine.InitOptions) (bool, error) {
 	v.Rootful = opts.Rootful
 
 	switch opts.ImagePath {
-	case Testing, Next, Stable, "":
+	// TODO these need to be re-typed as FCOSStreams
+	case machine.Testing.String(), machine.Next.String(), machine.Stable.String(), "":
 		// Get image as usual
 		v.ImageStream = opts.ImagePath
-		dd, err := machine.NewFcosDownloader(vmtype, v.Name, opts.ImagePath)
+		vp := GetVirtualizationProvider()
+		dd, err := machine.NewFcosDownloader(vmtype, v.Name, machine.FCOSStreamFromString(opts.ImagePath), vp)
 
 		if err != nil {
 			return false, err
@@ -302,30 +308,10 @@ func (v *MachineVM) Init(opts machine.InitOptions) (bool, error) {
 	mounts := []machine.Mount{}
 	for i, volume := range opts.Volumes {
 		tag := fmt.Sprintf("vol%d", i)
-		paths := strings.SplitN(volume, ":", 3)
-		source := paths[0]
-		target := source
-		readonly := false
-		securityModel := "mapped-xattr"
-		if len(paths) > 1 {
-			target = paths[1]
-		}
-		if len(paths) > 2 {
-			options := paths[2]
-			volopts := strings.Split(options, ",")
-			for _, o := range volopts {
-				switch {
-				case o == "rw":
-					readonly = false
-				case o == "ro":
-					readonly = true
-				case strings.HasPrefix(o, "security_model="):
-					securityModel = strings.Split(o, "=")[1]
-				default:
-					fmt.Printf("Unknown option: %s\n", o)
-				}
-			}
-		}
+		paths := pathsFromVolume(volume)
+		source := extractSourcePath(paths)
+		target := extractTargetPath(paths)
+		readonly, securityModel := extractMountOptions(paths)
 		if volumeType == VolumeTypeVirtfs {
 			virtfsOptions := fmt.Sprintf("local,path=%s,mount_tag=%s,security_model=%s", source, tag, securityModel)
 			if readonly {
@@ -391,11 +377,11 @@ func (v *MachineVM) Init(opts machine.InitOptions) (bool, error) {
 	// If the user provides an ignition file, we need to
 	// copy it into the conf dir
 	if len(opts.IgnitionPath) > 0 {
-		inputIgnition, err := ioutil.ReadFile(opts.IgnitionPath)
+		inputIgnition, err := os.ReadFile(opts.IgnitionPath)
 		if err != nil {
 			return false, err
 		}
-		return false, ioutil.WriteFile(v.getIgnitionFile(), inputIgnition, 0644)
+		return false, os.WriteFile(v.getIgnitionFile(), inputIgnition, 0644)
 	}
 	// Write the ignition file
 	ign := machine.DynamicIgnition{
@@ -406,7 +392,8 @@ func (v *MachineVM) Init(opts machine.InitOptions) (bool, error) {
 		WritePath: v.getIgnitionFile(),
 		UID:       v.UID,
 	}
-	err = machine.NewIgnitionFile(ign)
+
+	err = machine.NewIgnitionFile(ign, machine.QemuVirt)
 	return err == nil, err
 }
 
@@ -467,7 +454,7 @@ func (v *MachineVM) Set(_ string, opts machine.SetOptions) ([]error, error) {
 }
 
 // Start executes the qemu command line and forks it
-func (v *MachineVM) Start(name string, _ machine.StartOptions) error {
+func (v *MachineVM) Start(name string, opts machine.StartOptions) error {
 	var (
 		conn           net.Conn
 		err            error
@@ -560,18 +547,7 @@ func (v *MachineVM) Start(name string, _ machine.StartOptions) error {
 	attr.Files = files
 	cmdLine := v.CmdLine
 
-	// It is here for providing the ability to propagate
-	// proxy settings (e.g. HTTP_PROXY and others) on a start
-	// and avoid a need of re-creating/re-initiating a VM
-	if proxyOpts := machine.GetProxyVariables(); len(proxyOpts) > 0 {
-		proxyStr := "name=opt/com.coreos/environment,string="
-		var proxies string
-		for k, v := range proxyOpts {
-			proxies = fmt.Sprintf("%s%s=\"%s\"|", proxies, k, v)
-		}
-		proxyStr = fmt.Sprintf("%s%s", proxyStr, base64.StdEncoding.EncodeToString([]byte(proxies)))
-		cmdLine = append(cmdLine, "-fw_cfg", proxyStr)
-	}
+	cmdLine = propagateHostEnv(cmdLine)
 
 	// Disable graphic window when not in debug mode
 	// Done in start, so we're not suck with the debug level we used on init
@@ -613,7 +589,11 @@ func (v *MachineVM) Start(name string, _ machine.StartOptions) error {
 		}
 	}
 	defer cmd.Process.Release() //nolint:errcheck
-	fmt.Println("Waiting for VM ...")
+
+	if !opts.Quiet {
+		fmt.Println("Waiting for VM ...")
+	}
+
 	socketPath, err := getRuntimeDir()
 	if err != nil {
 		return err
@@ -659,7 +639,9 @@ func (v *MachineVM) Start(name string, _ machine.StartOptions) error {
 		}
 	}
 	for _, mount := range v.Mounts {
-		fmt.Printf("Mounting volume... %s:%s\n", mount.Source, mount.Target)
+		if !opts.Quiet {
+			fmt.Printf("Mounting volume... %s:%s\n", mount.Source, mount.Target)
+		}
 		// create mountpoint directory if it doesn't exist
 		// because / is immutable, we have to monkey around with permissions
 		// if we dont mount in /home or /mnt
@@ -692,10 +674,39 @@ func (v *MachineVM) Start(name string, _ machine.StartOptions) error {
 		}
 	}
 
-	v.waitAPIAndPrintInfo(forwardState, forwardSock)
+	v.waitAPIAndPrintInfo(forwardState, forwardSock, opts.NoInfo)
 	return nil
 }
 
+// propagateHostEnv is here for providing the ability to propagate
+// proxy and SSL settings (e.g. HTTP_PROXY and others) on a start
+// and avoid a need of re-creating/re-initiating a VM
+func propagateHostEnv(cmdLine []string) []string {
+	varsToPropagate := make([]string, 0)
+
+	for k, v := range machine.GetProxyVariables() {
+		varsToPropagate = append(varsToPropagate, fmt.Sprintf("%s=%q", k, v))
+	}
+
+	if sslCertFile, ok := os.LookupEnv("SSL_CERT_FILE"); ok {
+		pathInVM := filepath.Join(machine.UserCertsTargetPath, filepath.Base(sslCertFile))
+		varsToPropagate = append(varsToPropagate, fmt.Sprintf("%s=%q", "SSL_CERT_FILE", pathInVM))
+	}
+
+	if _, ok := os.LookupEnv("SSL_CERT_DIR"); ok {
+		varsToPropagate = append(varsToPropagate, fmt.Sprintf("%s=%q", "SSL_CERT_DIR", machine.UserCertsTargetPath))
+	}
+
+	if len(varsToPropagate) > 0 {
+		prefix := "name=opt/com.coreos/environment,string="
+		envVarsJoined := strings.Join(varsToPropagate, "|")
+		fwCfgArg := prefix + base64.StdEncoding.EncodeToString([]byte(envVarsJoined))
+		return append(cmdLine, "-fw_cfg", fwCfgArg)
+	}
+
+	return cmdLine
+}
+
 func (v *MachineVM) checkStatus(monitor *qmp.SocketMonitor) (machine.Status, error) {
 	// this is the format returned from the monitor
 	// {"return": {"status": "running", "singlestep": false, "running": true}}
@@ -1034,7 +1045,7 @@ func (v *MachineVM) SSH(_ string, opts machine.SSHOptions) error {
 	sshDestination := username + "@localhost"
 	port := strconv.Itoa(v.Port)
 
-	args := []string{"-i", v.IdentityPath, "-p", port, sshDestination, "-o", "UserKnownHostsFile=/dev/null",
+	args := []string{"-i", v.IdentityPath, "-p", port, sshDestination,
 		"-o", "StrictHostKeyChecking=no", "-o", "LogLevel=ERROR", "-o", "SetEnv=LC_ALL="}
 	if len(opts.Args) > 0 {
 		args = append(args, opts.Args...)
@@ -1093,7 +1104,7 @@ func getDiskSize(path string) (uint64, error) {
 }
 
 // List lists all vm's that use qemu virtualization
-func (p *Provider) List(_ machine.ListOptions) ([]*machine.ListResponse, error) {
+func (p *Virtualization) List(_ machine.ListOptions) ([]*machine.ListResponse, error) {
 	return getVMInfos()
 }
 
@@ -1109,7 +1120,7 @@ func getVMInfos() ([]*machine.ListResponse, error) {
 		vm := new(MachineVM)
 		if strings.HasSuffix(d.Name(), ".json") {
 			fullPath := filepath.Join(vmConfigDir, d.Name())
-			b, err := ioutil.ReadFile(fullPath)
+			b, err := os.ReadFile(fullPath)
 			if err != nil {
 				return err
 			}
@@ -1169,7 +1180,7 @@ func getVMInfos() ([]*machine.ListResponse, error) {
 	return listed, err
 }
 
-func (p *Provider) IsValidVMName(name string) (bool, error) {
+func (p *Virtualization) IsValidVMName(name string) (bool, error) {
 	infos, err := getVMInfos()
 	if err != nil {
 		return false, err
@@ -1184,10 +1195,10 @@ func (p *Provider) IsValidVMName(name string) (bool, error) {
 
 // CheckExclusiveActiveVM checks if there is a VM already running
 // that does not allow other VMs to be running
-func (p *Provider) CheckExclusiveActiveVM() (bool, string, error) {
+func (p *Virtualization) CheckExclusiveActiveVM() (bool, string, error) {
 	vms, err := getVMInfos()
 	if err != nil {
-		return false, "", fmt.Errorf("error checking VM active: %w", err)
+		return false, "", fmt.Errorf("checking VM active: %w", err)
 	}
 	for _, vm := range vms {
 		if vm.Running || vm.Starting {
@@ -1197,6 +1208,18 @@ func (p *Provider) CheckExclusiveActiveVM() (bool, string, error) {
 	return false, "", nil
 }
 
+func (p *Virtualization) Artifact() machine.Artifact {
+	return p.artifact
+}
+
+func (p *Virtualization) Compression() machine.ImageCompression {
+	return p.compression
+}
+
+func (p *Virtualization) Format() machine.ImageFormat {
+	return p.format
+}
+
 // startHostNetworking runs a binary on the host system that allows users
 // to set up port forwarding to the podman virtual machine
 func (v *MachineVM) startHostNetworking() (string, apiForwardingState, error) {
@@ -1313,7 +1336,7 @@ func (v *MachineVM) isIncompatible() bool {
 }
 
 func (v *MachineVM) userGlobalSocketLink() (string, error) {
-	path, err := machine.GetDataDir(v.Name)
+	path, err := machine.GetDataDir(machine.QemuVirt)
 	if err != nil {
 		logrus.Errorf("Resolving data dir: %s", err.Error())
 		return "", err
@@ -1324,7 +1347,7 @@ func (v *MachineVM) userGlobalSocketLink() (string, error) {
 
 func (v *MachineVM) forwardSocketPath() (*machine.VMFile, error) {
 	sockName := "podman.sock"
-	path, err := machine.GetDataDir(v.Name)
+	path, err := machine.GetDataDir(machine.QemuVirt)
 	if err != nil {
 		logrus.Errorf("Resolving data dir: %s", err.Error())
 		return nil, err
@@ -1384,7 +1407,7 @@ func (v *MachineVM) setPIDSocket() error {
 	return nil
 }
 
-// Deprecated: getSocketandPid is being replace by setPIDSocket and
+// Deprecated: getSocketandPid is being replaced by setPIDSocket and
 // machinefiles.
 func (v *MachineVM) getSocketandPid() (string, string, error) {
 	rtPath, err := getRuntimeDir()
@@ -1439,7 +1462,7 @@ func waitAndPingAPI(sock string) {
 	}
 }
 
-func (v *MachineVM) waitAPIAndPrintInfo(forwardState apiForwardingState, forwardSock string) {
+func (v *MachineVM) waitAPIAndPrintInfo(forwardState apiForwardingState, forwardSock string, noInfo bool) {
 	suffix := ""
 	if v.Name != machine.DefaultMachineName {
 		suffix = " " + v.Name
@@ -1467,38 +1490,41 @@ func (v *MachineVM) waitAPIAndPrintInfo(forwardState apiForwardingState, forward
 	}
 
 	waitAndPingAPI(forwardSock)
-	if !v.Rootful {
-		fmt.Printf("\nThis machine is currently configured in rootless mode. If your containers\n")
-		fmt.Printf("require root permissions (e.g. ports < 1024), or if you run into compatibility\n")
-		fmt.Printf("issues with non-podman clients, you can switch using the following command: \n")
-		fmt.Printf("\n\tpodman machine set --rootful%s\n\n", suffix)
-	}
 
-	fmt.Printf("API forwarding listening on: %s\n", forwardSock)
-	if forwardState == dockerGlobal {
-		fmt.Printf("Docker API clients default to this address. You do not need to set DOCKER_HOST.\n\n")
-	} else {
-		stillString := "still "
-		switch forwardState {
-		case notInstalled:
-			fmt.Printf("\nThe system helper service is not installed; the default Docker API socket\n")
-			fmt.Printf("address can't be used by podman. ")
-			if helper := findClaimHelper(); len(helper) > 0 {
-				fmt.Printf("If you would like to install it run the\nfollowing commands:\n")
-				fmt.Printf("\n\tsudo %s install\n", helper)
-				fmt.Printf("\tpodman machine stop%s; podman machine start%s\n\n", suffix, suffix)
-			}
-		case machineLocal:
-			fmt.Printf("\nAnother process was listening on the default Docker API socket address.\n")
-		case claimUnsupported:
-			fallthrough
-		default:
-			stillString = ""
+	if !noInfo {
+		if !v.Rootful {
+			fmt.Printf("\nThis machine is currently configured in rootless mode. If your containers\n")
+			fmt.Printf("require root permissions (e.g. ports < 1024), or if you run into compatibility\n")
+			fmt.Printf("issues with non-podman clients, you can switch using the following command: \n")
+			fmt.Printf("\n\tpodman machine set --rootful%s\n\n", suffix)
 		}
 
-		fmt.Printf("You can %sconnect Docker API clients by setting DOCKER_HOST using the\n", stillString)
-		fmt.Printf("following command in your terminal session:\n")
-		fmt.Printf("\n\texport DOCKER_HOST='unix://%s'\n\n", forwardSock)
+		fmt.Printf("API forwarding listening on: %s\n", forwardSock)
+		if forwardState == dockerGlobal {
+			fmt.Printf("Docker API clients default to this address. You do not need to set DOCKER_HOST.\n\n")
+		} else {
+			stillString := "still "
+			switch forwardState {
+			case notInstalled:
+				fmt.Printf("\nThe system helper service is not installed; the default Docker API socket\n")
+				fmt.Printf("address can't be used by podman. ")
+				if helper := findClaimHelper(); len(helper) > 0 {
+					fmt.Printf("If you would like to install it run the\nfollowing commands:\n")
+					fmt.Printf("\n\tsudo %s install\n", helper)
+					fmt.Printf("\tpodman machine stop%s; podman machine start%s\n\n", suffix, suffix)
+				}
+			case machineLocal:
+				fmt.Printf("\nAnother process was listening on the default Docker API socket address.\n")
+			case claimUnsupported:
+				fallthrough
+			default:
+				stillString = ""
+			}
+
+			fmt.Printf("You can %sconnect Docker API clients by setting DOCKER_HOST using the\n", stillString)
+			fmt.Printf("following command in your terminal session:\n")
+			fmt.Printf("\n\texport DOCKER_HOST='unix://%s'\n\n", forwardSock)
+		}
 	}
 }
 
@@ -1535,14 +1561,22 @@ func (v *MachineVM) writeConfig() error {
 		return err
 	}
 	// Write the JSON file
-	b, err := json.MarshalIndent(v, "", " ")
+	opts := &ioutils.AtomicFileWriterOptions{ExplicitCommit: true}
+	w, err := ioutils.NewAtomicFileWriterWithOpts(v.ConfigPath.GetPath(), 0644, opts)
 	if err != nil {
 		return err
 	}
-	if err := ioutil.WriteFile(v.ConfigPath.GetPath(), b, 0644); err != nil {
+	defer w.Close()
+
+	enc := json.NewEncoder(w)
+	enc.SetIndent("", " ")
+
+	if err := enc.Encode(v); err != nil {
 		return err
 	}
-	return nil
+
+	// Commit the changes to disk if no errors
+	return w.Commit()
 }
 
 // getImageFile wrapper returns the path to the image used
@@ -1643,7 +1677,7 @@ func (v *MachineVM) editCmdLine(flag string, value string) {
 }
 
 // RemoveAndCleanMachines removes all machine and cleans up any other files associated with podman machine
-func (p *Provider) RemoveAndCleanMachines() error {
+func (p *Virtualization) RemoveAndCleanMachines() error {
 	var (
 		vm             machine.VM
 		listResponse   []*machine.ListResponse
@@ -1690,7 +1724,7 @@ func (p *Provider) RemoveAndCleanMachines() error {
 		}
 		prevErr = err
 	} else {
-		err := os.RemoveAll(dataDir)
+		err := machine.GuardedRemoveAll(dataDir)
 		if err != nil {
 			if prevErr != nil {
 				logrus.Error(prevErr)
@@ -1707,7 +1741,7 @@ func (p *Provider) RemoveAndCleanMachines() error {
 		}
 		prevErr = err
 	} else {
-		err := os.RemoveAll(confDir)
+		err := machine.GuardedRemoveAll(confDir)
 		if err != nil {
 			if prevErr != nil {
 				logrus.Error(prevErr)
@@ -1718,7 +1752,7 @@ func (p *Provider) RemoveAndCleanMachines() error {
 	return prevErr
 }
 
-func (p *Provider) VMType() string {
+func (p *Virtualization) VMType() machine.VMType {
 	return vmtype
 }
 
@@ -1729,3 +1763,29 @@ func isRootful() bool {
 
 	return !rootless.IsRootless() && os.Getuid() != -1
 }
+
+func extractSourcePath(paths []string) string {
+	return paths[0]
+}
+
+func extractMountOptions(paths []string) (bool, string) {
+	readonly := false
+	securityModel := "none"
+	if len(paths) > 2 {
+		options := paths[2]
+		volopts := strings.Split(options, ",")
+		for _, o := range volopts {
+			switch {
+			case o == "rw":
+				readonly = false
+			case o == "ro":
+				readonly = true
+			case strings.HasPrefix(o, "security_model="):
+				securityModel = strings.Split(o, "=")[1]
+			default:
+				fmt.Printf("Unknown option: %s\n", o)
+			}
+		}
+	}
+	return readonly, securityModel
+}
