@@ -1,25 +1,27 @@
+//go:build windows
+// +build windows
+
 package wsl
 
 import (
 	"encoding/base64"
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"os"
+	"os/exec"
 	"path/filepath"
 	"strings"
 	"syscall"
 	"unicode/utf16"
 	"unsafe"
 
+	"github.com/containers/storage/pkg/homedir"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/windows"
 	"golang.org/x/sys/windows/registry"
-
-	"github.com/containers/storage/pkg/homedir"
 )
 
-//nolint
+// nolint
 type SHELLEXECUTEINFO struct {
 	cbSize         uint32
 	fMask          uint32
@@ -38,7 +40,7 @@ type SHELLEXECUTEINFO struct {
 	hProcess       syscall.Handle
 }
 
-//nolint
+// nolint
 type Luid struct {
 	lowPart  uint32
 	highPart int32
@@ -54,7 +56,7 @@ type TokenPrivileges struct {
 	privileges     [1]LuidAndAttributes
 }
 
-//nolint // Cleaner to refer to the official OS constant names, and consistent with syscall
+// nolint // Cleaner to refer to the official OS constant names, and consistent with syscall
 const (
 	SEE_MASK_NOCLOSEPROCESS         = 0x40
 	EWX_FORCEIFHUNG                 = 0x10
@@ -208,7 +210,7 @@ func reboot() error {
 		return fmt.Errorf("could not create data directory: %w", err)
 	}
 	commFile := filepath.Join(dataDir, "podman-relaunch.dat")
-	if err := ioutil.WriteFile(commFile, []byte(encoded), 0600); err != nil {
+	if err := os.WriteFile(commFile, []byte(encoded), 0600); err != nil {
 		return fmt.Errorf("could not serialize command state: %w", err)
 	}
 
@@ -216,7 +218,7 @@ func reboot() error {
 	if _, err := os.Lstat(filepath.Join(os.Getenv(localAppData), wtLocation)); err == nil {
 		wtCommand := wtPrefix + command
 		// RunOnce is limited to 260 chars (supposedly no longer in Builds >= 19489)
-		// For now fallbacak in cases of long usernames (>89 chars)
+		// For now fallback in cases of long usernames (>89 chars)
 		if len(wtCommand) < 260 {
 			command = wtCommand
 		}
@@ -262,19 +264,19 @@ func obtainShutdownPrivilege() error {
 
 	var hToken uintptr
 	if ret, _, err := OpenProcessToken.Call(uintptr(proc), TOKEN_ADJUST_PRIVILEGES|TOKEN_QUERY, uintptr(unsafe.Pointer(&hToken))); ret != 1 {
-		return fmt.Errorf("error opening process token: %w", err)
+		return fmt.Errorf("opening process token: %w", err)
 	}
 
 	var privs TokenPrivileges
 	if ret, _, err := LookupPrivilegeValue.Call(uintptr(0), uintptr(unsafe.Pointer(syscall.StringToUTF16Ptr(SeShutdownName))), uintptr(unsafe.Pointer(&(privs.privileges[0].luid)))); ret != 1 {
-		return fmt.Errorf("error looking up shutdown privilege: %w", err)
+		return fmt.Errorf("looking up shutdown privilege: %w", err)
 	}
 
 	privs.privilegeCount = 1
 	privs.privileges[0].attributes = SE_PRIVILEGE_ENABLED
 
 	if ret, _, err := AdjustTokenPrivileges.Call(hToken, 0, uintptr(unsafe.Pointer(&privs)), 0, uintptr(0), 0); ret != 1 {
-		return fmt.Errorf("error enabling shutdown privilege on token: %w", err)
+		return fmt.Errorf("enabling shutdown privilege on token: %w", err)
 	}
 
 	return nil
@@ -343,3 +345,17 @@ func sendQuit(tid uint32) {
 	postMessage := user32.NewProc("PostThreadMessageW")
 	postMessage.Call(uintptr(tid), WM_QUIT, 0, 0)
 }
+
+func SilentExec(command string, args ...string) error {
+	cmd := exec.Command(command, args...)
+	cmd.SysProcAttr = &syscall.SysProcAttr{CreationFlags: 0x08000000}
+	cmd.Stdout = nil
+	cmd.Stderr = nil
+	return cmd.Run()
+}
+
+func SilentExecCmd(command string, args ...string) *exec.Cmd {
+	cmd := exec.Command(command, args...)
+	cmd.SysProcAttr = &syscall.SysProcAttr{CreationFlags: 0x08000000}
+	return cmd
+}
