@@ -6,7 +6,7 @@ import (
 	"sort"
 	"strings"
 
-	. "github.com/containers/podman/v3/test/utils"
+	. "github.com/containers/podman/v4/test/utils"
 	"github.com/docker/go-units"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
@@ -27,7 +27,6 @@ var _ = Describe("Podman images", func() {
 		}
 		podmanTest = PodmanTestCreate(tempdir)
 		podmanTest.Setup()
-		podmanTest.SeedImages()
 	})
 
 	AfterEach(func() {
@@ -41,8 +40,8 @@ var _ = Describe("Podman images", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 		Expect(len(session.OutputToStringArray())).To(BeNumerically(">", 2))
-		Expect(session.LineInOutputStartsWith("quay.io/libpod/alpine")).To(BeTrue())
-		Expect(session.LineInOutputStartsWith("quay.io/libpod/busybox")).To(BeTrue())
+		Expect(session.OutputToStringArray()).To(ContainElement(HavePrefix("quay.io/libpod/alpine")))
+		Expect(session.OutputToStringArray()).To(ContainElement(HavePrefix("quay.io/libpod/busybox")))
 	})
 
 	It("podman image List", func() {
@@ -50,8 +49,8 @@ var _ = Describe("Podman images", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 		Expect(len(session.OutputToStringArray())).To(BeNumerically(">", 2))
-		Expect(session.LineInOutputStartsWith("quay.io/libpod/alpine")).To(BeTrue())
-		Expect(session.LineInOutputStartsWith("quay.io/libpod/busybox")).To(BeTrue())
+		Expect(session.OutputToStringArray()).To(ContainElement(HavePrefix("quay.io/libpod/alpine")))
+		Expect(session.OutputToStringArray()).To(ContainElement(HavePrefix("quay.io/libpod/busybox")))
 	})
 
 	It("podman images with multiple tags", func() {
@@ -78,7 +77,7 @@ var _ = Describe("Podman images", func() {
 		session = podmanTest.Podman([]string{"images", "-qn"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(len(session.OutputToStringArray())).To(BeNumerically("==", len(CACHE_IMAGES)))
+		Expect(session.OutputToStringArray()).To(HaveLen(len(CACHE_IMAGES)))
 	})
 
 	It("podman images with digests", func() {
@@ -86,22 +85,22 @@ var _ = Describe("Podman images", func() {
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 		Expect(len(session.OutputToStringArray())).To(BeNumerically(">", 2))
-		Expect(session.LineInOutputStartsWith("quay.io/libpod/alpine")).To(BeTrue())
-		Expect(session.LineInOutputStartsWith("quay.io/libpod/busybox")).To(BeTrue())
+		Expect(session.OutputToStringArray()).To(ContainElement(HavePrefix("quay.io/libpod/alpine")))
+		Expect(session.OutputToStringArray()).To(ContainElement(HavePrefix("quay.io/libpod/busybox")))
 	})
 
 	It("podman empty images list in JSON format", func() {
 		session := podmanTest.Podman([]string{"images", "--format=json", "not-existing-image"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(session.IsJSONOutputValid()).To(BeTrue())
+		Expect(session.OutputToString()).To(BeValidJSON())
 	})
 
 	It("podman images in JSON format", func() {
 		session := podmanTest.Podman([]string{"images", "--format=json"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(session.IsJSONOutputValid()).To(BeTrue())
+		Expect(session.OutputToString()).To(BeValidJSON())
 	})
 
 	It("podman images in GO template format", func() {
@@ -125,7 +124,7 @@ var _ = Describe("Podman images", func() {
 		session := podmanTest.Podman([]string{"images", "-q", ALPINE})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(len(session.OutputToStringArray())).To(Equal(1))
+		Expect(session.OutputToStringArray()).To(HaveLen(1))
 
 		session = podmanTest.Podman([]string{"tag", ALPINE, "foo:a"})
 		session.WaitWithDefaultTimeout()
@@ -137,43 +136,42 @@ var _ = Describe("Podman images", func() {
 		session = podmanTest.Podman([]string{"images", "-q", "foo"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(len(session.OutputToStringArray())).To(Equal(2))
+		Expect(session.OutputToStringArray()).To(HaveLen(2))
 	})
 
 	It("podman images filter reference", func() {
-		result := podmanTest.Podman([]string{"images", "-q", "-f", "reference=quay.io*"})
+		result := podmanTest.Podman([]string{"images", "-q", "-f", "reference=quay.io/libpod/*"})
 		result.WaitWithDefaultTimeout()
 		Expect(result).Should(Exit(0))
-		Expect(len(result.OutputToStringArray())).To(Equal(8))
+		Expect(result.OutputToStringArray()).To(HaveLen(8))
 
-		retalpine := podmanTest.Podman([]string{"images", "-f", "reference=a*pine"})
+		retalpine := podmanTest.Podman([]string{"images", "-f", "reference=*lpine*"})
 		retalpine.WaitWithDefaultTimeout()
 		Expect(retalpine).Should(Exit(0))
-		Expect(len(retalpine.OutputToStringArray())).To(Equal(6))
-		Expect(retalpine.LineInOutputContains("alpine")).To(BeTrue())
+		Expect(retalpine.OutputToStringArray()).To(HaveLen(6))
+		Expect(retalpine.OutputToString()).To(ContainSubstring("alpine"))
 
 		retalpine = podmanTest.Podman([]string{"images", "-f", "reference=alpine"})
 		retalpine.WaitWithDefaultTimeout()
 		Expect(retalpine).Should(Exit(0))
-		Expect(len(retalpine.OutputToStringArray())).To(Equal(6))
-		Expect(retalpine.LineInOutputContains("alpine")).To(BeTrue())
+		Expect(retalpine.OutputToStringArray()).To(HaveLen(2))
+		Expect(retalpine.OutputToString()).To(ContainSubstring("alpine"))
 
 		retnone := podmanTest.Podman([]string{"images", "-q", "-f", "reference=bogus"})
 		retnone.WaitWithDefaultTimeout()
 		Expect(retnone).Should(Exit(0))
-		Expect(len(retnone.OutputToStringArray())).To(Equal(0))
+		Expect(retnone.OutputToStringArray()).To(BeEmpty())
 	})
 
 	It("podman images filter before image", func() {
 		dockerfile := `FROM quay.io/libpod/alpine:latest
-RUN apk update && apk add strace
+RUN echo hello > /hello
 `
 		podmanTest.BuildImage(dockerfile, "foobar.com/before:latest", "false")
 		result := podmanTest.Podman([]string{"images", "-q", "-f", "before=foobar.com/before:latest"})
 		result.WaitWithDefaultTimeout()
 		Expect(result).Should(Exit(0))
-		Expect(len(result.OutputToStringArray()) >= 1).To(BeTrue())
-
+		Expect(result.OutputToStringArray()).ToNot(BeEmpty())
 	})
 
 	It("podman images workingdir from  image", func() {
@@ -187,24 +185,24 @@ WORKDIR /test
 		Expect(result.OutputToString()).To(Equal("/test"))
 	})
 
-	It("podman images filter since image", func() {
-		dockerfile := `FROM quay.io/libpod/alpine:latest
+	It("podman images filter since/after image", func() {
+		dockerfile := `FROM scratch
 `
-		podmanTest.BuildImage(dockerfile, "foobar.com/before:latest", "false")
-		result := podmanTest.Podman([]string{"images", "-q", "-f", "since=quay.io/libpod/alpine:latest"})
-		result.WaitWithDefaultTimeout()
-		Expect(result).Should(Exit(0))
-		Expect(len(result.OutputToStringArray())).To(Equal(9))
-	})
+		podmanTest.BuildImage(dockerfile, "foobar.com/one:latest", "false")
+		podmanTest.BuildImage(dockerfile, "foobar.com/two:latest", "false")
+		podmanTest.BuildImage(dockerfile, "foobar.com/three:latest", "false")
 
-	It("podman image list filter after image", func() {
-		dockerfile := `FROM quay.io/libpod/alpine:latest
-`
-		podmanTest.BuildImage(dockerfile, "foobar.com/before:latest", "false")
-		result := podmanTest.Podman([]string{"image", "list", "-q", "-f", "after=quay.io/libpod/alpine:latest"})
+		// `since` filter
+		result := podmanTest.PodmanNoCache([]string{"images", "-q", "-f", "since=foobar.com/one:latest"})
 		result.WaitWithDefaultTimeout()
 		Expect(result).Should(Exit(0))
-		Expect(result.OutputToStringArray()).Should(HaveLen(9), "list filter output: %q", result.OutputToString())
+		Expect(result.OutputToStringArray()).To(HaveLen(2))
+
+		// `after` filter
+		result = podmanTest.Podman([]string{"image", "list", "-q", "-f", "after=foobar.com/one:latest"})
+		result.WaitWithDefaultTimeout()
+		Expect(result).Should(Exit(0))
+		Expect(result.OutputToStringArray()).Should(HaveLen(2), "list filter output: %q", result.OutputToString())
 	})
 
 	It("podman images filter dangling", func() {
@@ -219,7 +217,8 @@ WORKDIR /test
 	})
 
 	It("podman pull by digest and list --all", func() {
-		// Prevent regressing on issue #7651.
+		// Prevent regressing on issue #7651: error parsing name that includes a digest
+		// component as if were a name that includes tag component.
 		digestPullAndList := func(noneTag bool) {
 			session := podmanTest.Podman([]string{"pull", ALPINEAMD64DIGEST})
 			session.WaitWithDefaultTimeout()
@@ -229,21 +228,24 @@ WORKDIR /test
 			result.WaitWithDefaultTimeout()
 			Expect(result).Should(Exit(0))
 
-			found, _ := result.GrepString("<none>")
 			if noneTag {
-				Expect(found).To(BeTrue())
+				Expect(result.OutputToString()).To(ContainSubstring("<none>"))
 			} else {
-				Expect(found).To(BeFalse())
+				Expect(result.OutputToString()).To(Not(ContainSubstring("<none>")))
 			}
 		}
-		// No "<none>" tag as tagged alpine instances should be present.
+		// No "<none>" in the tag column as image tagged as "ALPINE" should be present in
+		// the additional image store we're using.  Pull the same image by another name to
+		// copy an entry for the image into read-write storage so that the name can be
+		// attached to it.
 		session := podmanTest.Podman([]string{"pull", ALPINELISTTAG})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 		digestPullAndList(false)
 
-		// Now remove all images, re-pull by digest and check for the "<none>" tag.
-		session = podmanTest.Podman([]string{"rmi", "-af"})
+		// Now remove all names from the read-write image record, re-pull by digest and
+		// check for the "<none>" in its listing.
+		session = podmanTest.Podman([]string{"untag", ALPINELISTTAG})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
 
@@ -254,7 +256,7 @@ WORKDIR /test
 		session := podmanTest.Podman([]string{"inspect", "--format=json", ALPINE})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(session.IsJSONOutputValid()).To(BeTrue())
+		Expect(session.OutputToString()).To(BeValidJSON())
 		imageData := session.InspectImageJSON()
 
 		result := podmanTest.Podman([]string{"images", "sha256:" + imageData[0].ID})
@@ -266,7 +268,7 @@ WORKDIR /test
 		session := podmanTest.Podman([]string{"image", "inspect", "--format=json", ALPINE})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(session.IsJSONOutputValid()).To(BeTrue())
+		Expect(session.OutputToString()).To(BeValidJSON())
 		imageData := session.InspectImageJSON()
 
 		result := podmanTest.Podman([]string{"image", "ls", fmt.Sprintf("sha256:%s", imageData[0].ID)})
@@ -327,12 +329,12 @@ ENV foo=bar
 		session := podmanTest.Podman([]string{"images"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(len(session.OutputToStringArray())).To(Equal(len(CACHE_IMAGES) + 2))
+		Expect(session.OutputToStringArray()).To(HaveLen(len(CACHE_IMAGES) + 2))
 
 		session2 := podmanTest.Podman([]string{"images", "--all"})
 		session2.WaitWithDefaultTimeout()
 		Expect(session2).Should(Exit(0))
-		Expect(len(session2.OutputToStringArray())).To(Equal(len(CACHE_IMAGES) + 4))
+		Expect(session2.OutputToStringArray()).To(HaveLen(len(CACHE_IMAGES) + 4))
 	})
 
 	It("podman images filter by label", func() {
@@ -344,7 +346,7 @@ LABEL "com.example.vendor"="Example Vendor"
 		session := podmanTest.Podman([]string{"images", "-f", "label=version=1.0"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(len(session.OutputToStringArray())).To(Equal(2))
+		Expect(session.OutputToStringArray()).To(HaveLen(2))
 	})
 
 	It("podman with images with no layers", func() {
@@ -381,7 +383,7 @@ LABEL "com.example.vendor"="Example Vendor"
 		session = podmanTest.Podman([]string{"history", "--quiet", "foo"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(len(session.OutputToStringArray())).To(Equal(6))
+		Expect(session.OutputToStringArray()).To(HaveLen(6))
 
 		session = podmanTest.Podman([]string{"image", "list", "foo"})
 		session.WaitWithDefaultTimeout()
@@ -425,4 +427,46 @@ LABEL "com.example.vendor"="Example Vendor"
 		Expect(result.OutputToStringArray()).To(Not(Equal(result1.OutputToStringArray())))
 	})
 
+	It("podman image prune --filter", func() {
+		dockerfile := `FROM quay.io/libpod/alpine:latest
+RUN > file
+`
+		dockerfile2 := `FROM quay.io/libpod/alpine:latest
+RUN > file2
+`
+		podmanTest.BuildImageWithLabel(dockerfile, "foobar.com/workdir:latest", "false", "abc")
+		podmanTest.BuildImageWithLabel(dockerfile2, "foobar.com/workdir:latest", "false", "xyz")
+		// --force used to avoid y/n question
+		result := podmanTest.Podman([]string{"image", "prune", "--filter", "label=abc", "--force"})
+		result.WaitWithDefaultTimeout()
+		Expect(result).Should(Exit(0))
+		Expect(result.OutputToStringArray()).To(HaveLen(1))
+
+		// check if really abc is removed
+		result = podmanTest.Podman([]string{"image", "list", "--filter", "label=abc"})
+		Expect(result.OutputToStringArray()).To(BeEmpty())
+
+	})
+
+	It("podman builder prune", func() {
+		dockerfile := `FROM quay.io/libpod/alpine:latest
+RUN > file
+`
+		dockerfile2 := `FROM quay.io/libpod/alpine:latest
+RUN > file2
+`
+		podmanTest.BuildImageWithLabel(dockerfile, "foobar.com/workdir:latest", "false", "abc")
+		podmanTest.BuildImageWithLabel(dockerfile2, "foobar.com/workdir:latest", "false", "xyz")
+		// --force used to to avoid y/n question
+		result := podmanTest.Podman([]string{"builder", "prune", "--filter", "label=abc", "--force"})
+		result.WaitWithDefaultTimeout()
+		Expect(result).Should(Exit(0))
+		Expect(result.OutputToStringArray()).To(HaveLen(1))
+
+		// check if really abc is removed
+		result = podmanTest.Podman([]string{"image", "list", "--filter", "label=abc"})
+		Expect(result.OutputToStringArray()).To(BeEmpty())
+
+	})
+
 })
